# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-30 11:45
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0065_auto_20170930_1249'),
    ]

    operations = [
        migrations.AddField(
            model_name='collectionpluginmodel',
            name='limit',
            field=models.PositiveIntegerField(default=6, verbose_name='Limit'),
        ),
    ]
