# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-29 14:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0049_remove_untranslated_fields_from_collection'),
    ]

    operations = [
        migrations.RenameField(
            model_name='collectiontranslation',
            old_name='description_t',
            new_name='description',
        ),
        migrations.RenameField(
            model_name='collectiontranslation',
            old_name='meta_description_t',
            new_name='meta_description',
        ),
        migrations.RenameField(
            model_name='collectiontranslation',
            old_name='meta_keywords_t',
            new_name='meta_keywords',
        ),
        migrations.RenameField(
            model_name='collectiontranslation',
            old_name='meta_text_t',
            new_name='meta_text',
        ),
        migrations.RenameField(
            model_name='collectiontranslation',
            old_name='page_title_t',
            new_name='page_title',
        ),
        migrations.RenameField(
            model_name='collectiontranslation',
            old_name='title_t',
            new_name='title',
        ),
    ]
