# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-01 13:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0067_auto_20170930_2021'),
    ]

    operations = [
        migrations.AddField(
            model_name='collectionpluginmodel',
            name='ajax_load',
            field=models.BooleanField(default=False, verbose_name='AJAX load'),
        ),
    ]
