# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-27 12:35
from __future__ import unicode_literals

import cms.models.fields
from django.db import migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('pcart_catalog', '0022_auto_20170814_1258'),
    ]

    operations = [
        migrations.AddField(
            model_name='producttype',
            name='primary_placeholder',
            field=cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='product_type_primary', slotname='primary_product_type_placeholder', to='cms.Placeholder'),
        ),
        migrations.AddField(
            model_name='producttype',
            name='secondary_placeholder',
            field=cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='product_type_secondary', slotname='secondary_product_type_placeholder', to='cms.Placeholder'),
        ),
    ]
