# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-27 16:59
from __future__ import unicode_literals

import cms.models.fields
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('pcart_catalog', '0024_producttypeproperty_hidden'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='condition_max_price',
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=10, verbose_name='Max price'),
        ),
        migrations.AddField(
            model_name='collection',
            name='condition_min_price',
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=10, verbose_name='Min price'),
        ),
        migrations.AddField(
            model_name='collection',
            name='condition_product_types',
            field=models.ManyToManyField(blank=True, related_name='auto_collections', to='pcart_catalog.ProductType', verbose_name='Product types'),
        ),
        migrations.AddField(
            model_name='collection',
            name='condition_vendors',
            field=models.ManyToManyField(blank=True, related_name='auto_collections', to='pcart_catalog.Vendor', verbose_name='Vendors'),
        ),
        migrations.AddField(
            model_name='collection',
            name='exlude_tags',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=30), blank=True, default=list, size=None, verbose_name='Exclude tags'),
        ),
        migrations.AddField(
            model_name='collection',
            name='extra_placeholder',
            field=cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='collection_extra', slotname='secondary_collection_placeholder', to='cms.Placeholder'),
        ),
        migrations.AddField(
            model_name='collection',
            name='mandatory_tags',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=30), blank=True, default=list, size=None, verbose_name='Mandatory tags'),
        ),
        migrations.AddField(
            model_name='collection',
            name='optional_tags',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=30), blank=True, default=list, size=None, verbose_name='Optional tags'),
        ),
        migrations.AddField(
            model_name='collection',
            name='primary_placeholder',
            field=cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='collection_primary', slotname='primary_collection_placeholder', to='cms.Placeholder'),
        ),
        migrations.AddField(
            model_name='collection',
            name='products_selection',
            field=models.CharField(default='manual', help_text='Use Automatic mode if you want to select products based on conditions.', max_length=30, verbose_name='Products selection'),
        ),
        migrations.AddField(
            model_name='collection',
            name='secondary_placeholder',
            field=cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='collection_secondary', slotname='secondary_collection_placeholder', to='cms.Placeholder'),
        ),
    ]
