# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-28 09:51
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0032_auto_20170927_2141'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='extra_data',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, verbose_name='Extra data'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_float_1',
            field=models.FloatField(default=0.0, verbose_name='Extra float 1'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_float_2',
            field=models.FloatField(default=0.0, verbose_name='Extra float 2'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_id_1',
            field=models.CharField(blank=True, db_index=True, default='', max_length=255, verbose_name='Extra ID 1'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_id_2',
            field=models.CharField(blank=True, db_index=True, default='', max_length=255, verbose_name='Extra ID 2'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_int_1',
            field=models.IntegerField(default=0, verbose_name='Extra int 1'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_int_2',
            field=models.IntegerField(default=0, verbose_name='Extra int 2'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_price_1',
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=10, verbose_name='Extra price 1'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_price_2',
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=10, verbose_name='Extra price 2'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_price_3',
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=10, verbose_name='Extra price 3'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_text_1',
            field=models.TextField(blank=True, default='', verbose_name='Extra text 1'),
        ),
        migrations.AddField(
            model_name='product',
            name='extra_text_2',
            field=models.TextField(blank=True, default='', verbose_name='Extra text 2'),
        ),
        migrations.AddField(
            model_name='product',
            name='rank_1',
            field=models.FloatField(default=0.0, verbose_name='Rank 1'),
        ),
        migrations.AddField(
            model_name='product',
            name='rank_2',
            field=models.FloatField(default=0.0, verbose_name='Rank 2'),
        ),
        migrations.AddField(
            model_name='product',
            name='rank_3',
            field=models.FloatField(default=0.0, verbose_name='Rank 3'),
        ),
        migrations.AddField(
            model_name='producttype',
            name='default_quantity_for_purchase',
            field=models.PositiveIntegerField(default=1, verbose_name='Default quantity for purchase'),
        ),
    ]
