# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-29 18:19
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0059_rename_translatable_fields_for_productvariant'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProductImageTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('title_t', models.CharField(blank=True, default='', max_length=255, verbose_name='Title')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='pcart_catalog.ProductImage')),
            ],
            options={
                'db_tablespace': '',
                'db_table': 'pcart_catalog_productimage_translation',
                'verbose_name': 'Product image Translation',
                'default_permissions': (),
                'managed': True,
            },
        ),
        migrations.AlterUniqueTogether(
            name='productimagetranslation',
            unique_together=set([('language_code', 'master')]),
        ),
    ]
