# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-14 09:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0021_collection_custom_url_filter_rules'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='price_varies',
            field=models.BooleanField(default=False, verbose_name='Price varies'),
        ),
        migrations.AddField(
            model_name='productvariant',
            name='price_varies',
            field=models.BooleanField(default=False, verbose_name='Price varies'),
        ),
    ]
