# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-30 17:21
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0066_collectionpluginmodel_limit'),
    ]

    operations = [
        migrations.AddField(
            model_name='collectionpluginmodel',
            name='data',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, help_text='Optional. Some templates may support additional options for conditional rendering.', verbose_name='Data'),
        ),
        migrations.AddField(
            model_name='collectionpluginmodel',
            name='show_title',
            field=models.BooleanField(default=True, verbose_name='Show title'),
        ),
        migrations.AddField(
            model_name='collectionpluginmodel',
            name='title',
            field=models.CharField(blank=True, default='', help_text='Optional. If not set the collection title will be used.', max_length=255, verbose_name='Title'),
        ),
    ]
