# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-27 18:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0028_auto_20170927_2047'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='meta_keywords',
            field=models.TextField(blank=True, verbose_name='Meta keywords'),
        ),
        migrations.AddField(
            model_name='collection',
            name='meta_text',
            field=models.TextField(blank=True, verbose_name='Meta text'),
        ),
        migrations.AddField(
            model_name='product',
            name='meta_keywords',
            field=models.TextField(blank=True, verbose_name='Meta keywords'),
        ),
        migrations.AddField(
            model_name='product',
            name='meta_text',
            field=models.TextField(blank=True, verbose_name='Meta text'),
        ),
    ]
