# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-27 17:06
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0026_auto_20170927_2003'),
    ]

    operations = [
        migrations.RenameField(
            model_name='collection',
            old_name='exlude_tags',
            new_name='exclude_tags',
        ),
    ]
