# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-30 09:47
from __future__ import unicode_literals

from django.db import migrations
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0063_rename_translatable_fields_for_productimage'),
    ]

    operations = [
        migrations.AlterField(
            model_name='collectiontranslation',
            name='description',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name='Description'),
        ),
    ]
