# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-27 18:41
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0031_vendor_description'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='productstatus',
            options={'ordering': ['-weight'], 'verbose_name': 'Product status', 'verbose_name_plural': 'Product statuses'},
        ),
    ]
