# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-26 17:18
from __future__ import unicode_literals

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0012_remove_collection_exclude_properties_filters'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='exclude_properties_filters',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=70), blank=True, default=list, help_text='Comma separated list of properties labels which you want to exclude from filters.', size=None, verbose_name='Exclude properties filters'),
        ),
    ]
