# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-26 16:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0009_collectionpluginmodel'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='exclude_properties_filters',
            field=models.CharField(default='', help_text='Comma separated list of properties tags which you want to exclude from filters.', max_length=255, verbose_name='Exclude properties filters'),
        ),
        migrations.AddField(
            model_name='collection',
            name='show_price_filter',
            field=models.BooleanField(default=True, verbose_name='Show price filter'),
        ),
        migrations.AddField(
            model_name='collection',
            name='show_properties_filters',
            field=models.BooleanField(default=True, verbose_name='Show properties filters'),
        ),
        migrations.AddField(
            model_name='collection',
            name='show_vendor_filter',
            field=models.BooleanField(default=True, verbose_name='Show vendor filter'),
        ),
        migrations.AlterField(
            model_name='collection',
            name='url_filter_rules',
            field=models.TextField(blank=True, default='VENDOR\nPRICE', help_text='Use separate lines for different rules. See documentation for details.', verbose_name='URL filter rules'),
        ),
    ]
