# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-02 07:54
from __future__ import unicode_literals

import django.contrib.postgres.fields
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('pcart_catalog', '0068_collectionpluginmodel_ajax_load'),
    ]

    operations = [
        migrations.CreateModel(
            name='SimilarProductsPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='pcart_catalog_similarproductspluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('show_title', models.BooleanField(default=True, verbose_name='Show title')),
                ('title', models.CharField(blank=True, default='', help_text='Optional. If not set the collection title will be used.', max_length=255, verbose_name='Title')),
                ('tags_or_prefixes', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=30), blank=True, default=list, help_text='You can specify like full tag as `prefix:value` but also `prefix` only.', size=None, verbose_name='Tags or prefixes')),
                ('vendor_filtering', models.BooleanField(default=False, verbose_name='Vendor filtering')),
                ('price_difference_filtering', models.CharField(blank=True, choices=[('', 'Do not filter'), ('absolute', 'Absolute difference'), ('relative', 'Relative difference')], default='', max_length=30, verbose_name='Price difference filtering')),
                ('price_filtering_value', models.DecimalField(decimal_places=2, default=0.0, max_digits=10, verbose_name='Price filtering value')),
                ('sorting', models.CharField(blank=True, default='', max_length=100, verbose_name='Sorting')),
                ('limit', models.PositiveIntegerField(default=6, verbose_name='Limit')),
                ('template_name', models.CharField(blank=True, default='', max_length=100, verbose_name='Template name')),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, help_text='Optional. Some templates may support additional options for conditional rendering.', verbose_name='Data')),
                ('ajax_load', models.BooleanField(default=False, verbose_name='AJAX load')),
                ('collection', mptt.fields.TreeForeignKey(on_delete=django.db.models.deletion.CASCADE, to='pcart_catalog.Collection', verbose_name='Collection')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]
