# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-09 11:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0007_auto_20170309_1144'),
    ]

    operations = [
        migrations.AddField(
            model_name='productimage',
            name='downloaded',
            field=models.BooleanField(default=False, verbose_name='Downloaded'),
        ),
    ]
