# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-10 12:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0020_auto_20170808_1104'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='custom_url_filter_rules',
            field=models.BooleanField(default=False, verbose_name='Custom URL filter rules'),
        ),
    ]
