# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-04-11 07:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0013_collection_exclude_properties_filters'),
    ]

    operations = [
        migrations.AlterField(
            model_name='collectionpluginmodel',
            name='collection',
            field=mptt.fields.TreeForeignKey(on_delete=django.db.models.deletion.CASCADE, to='pcart_catalog.Collection', verbose_name='Collection'),
        ),
        migrations.AlterField(
            model_name='collectionpluginmodel',
            name='filter_string',
            field=models.CharField(blank=True, default='', help_text='Use the list of filter chunks separated with / and ; characters.', max_length=255, verbose_name='Filter string'),
        ),
        migrations.AlterField(
            model_name='collectionpluginmodel',
            name='sorting',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='Sorting'),
        ),
        migrations.AlterField(
            model_name='collectionpluginmodel',
            name='template_name',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='Template name'),
        ),
    ]
