# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-29 13:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0040_migrate_translatable_fields_for_productstatus'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='productstatus',
            name='title',
        ),
    ]
