# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-08 08:04
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0019_auto_20170505_1202'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='external_id',
            field=models.CharField(blank=True, db_index=True, default='', max_length=255, verbose_name='External ID'),
        ),
        migrations.AddField(
            model_name='product',
            name='quantity',
            field=models.PositiveIntegerField(blank=True, default=None, null=True, verbose_name='Quantity'),
        ),
        migrations.AddField(
            model_name='productvariant',
            name='external_id',
            field=models.CharField(blank=True, db_index=True, default='', max_length=255, verbose_name='External ID'),
        ),
        migrations.AddField(
            model_name='productvariant',
            name='quantity',
            field=models.PositiveIntegerField(blank=True, default=None, null=True, verbose_name='Quantity'),
        ),
    ]
