# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-20 18:47
from __future__ import unicode_literals

import adminsortable.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0004_auto_20170220_2032'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='productimage',
            options={'ordering': ['position'], 'verbose_name': 'Product image', 'verbose_name_plural': 'Product images'},
        ),
        migrations.AlterField(
            model_name='productimage',
            name='position',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False, verbose_name='Position'),
        ),
        migrations.AlterField(
            model_name='productimage',
            name='product',
            field=adminsortable.fields.SortableForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='images', to='pcart_catalog.Product', verbose_name='Product'),
        ),
        migrations.AlterField(
            model_name='producttypeproperty',
            name='position',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False, verbose_name='Position'),
        ),
        migrations.AlterField(
            model_name='producttypeproperty',
            name='product_type',
            field=adminsortable.fields.SortableForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='properties', to='pcart_catalog.ProductType', verbose_name='Product type'),
        ),
    ]
