# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-27 12:40
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0023_auto_20170927_1535'),
    ]

    operations = [
        migrations.AddField(
            model_name='producttypeproperty',
            name='hidden',
            field=models.BooleanField(default=False, help_text='Useful for hide some product specified information from a web page.', verbose_name='Hidden'),
        ),
    ]
