# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-27 17:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0025_auto_20170927_1959'),
    ]

    operations = [
        migrations.AlterField(
            model_name='collection',
            name='products_selection',
            field=models.CharField(choices=[('manual', 'Manual'), ('automatic', 'Automatic')], default='manual', help_text='Use Automatic mode if you want to select products based on conditions.', max_length=30, verbose_name='Products selection'),
        ),
    ]
